#!/usr/bin/perl
# 
# $Revision: 1.26 $
# Copyright 2011 Teleflora.
#
# The script constitutes the postpatch script for the process of
# applying patches to Daisy systems and for converting a Daisy
# system from the Global East payment system to Elavon.
#
# Called from applypatch.pl with arguments of:
#
#	ARGV[0] = path to current daisy db dir
#	ARGV[1] = time stamp
#	ARGV[2] = list of patch files (optional)
#

use strict;
use warnings;
use Getopt::Long;
use POSIX;
use File::Basename;

my $CVS_REVISION = '$Revision: 1.26 $';
my $PROGNAME = basename($0);

#
# Command line options
#
my $HELP = 0;
my $VERSION = 0;
my $DAISY = 0;
my $DAISY_DIR = '/d/daisy';
my $EDIRUPDATE = 1;
my $CONVERT_TO_ELAVON = 0;
my $INSTALL_FTD_FILES = 0;
my $DEBUG = 0;

#
# Alternate os and root for testing.
#
my $ALTROOT = "";
my $ALTOS = "";

#
# Global variables
#
my $OS = "";
my $POSDIR = "";
my $LOGFILENAME = "log/RTI-Patches.log";
my $PATCH_VERSION = "";
my @NU_BINARIES = ();
my @NU_DATAFILES = ();
my $TIMESTAMP = "";
my $DAISY_FTD_SUBS_FILE = "daisy-ftd-subs.txt";
my $APPLYPATCH_CONFIG_FILE = "applypatch.config";
my @CONFIG_QA_SHOPCODE = ();

#
# Constants
#

# Exit status values
my $EXIT_OK = 0;
my $EXIT_ERROR = -1;
my $EXIT_COMMAND_LINE = 1;
my $EXIT_MUST_BE_ROOT = 2;
my $EXIT_PATCH_VALIDATION = 3;
my $EXIT_POSTPATCH_CLEANUP = 8;
my $EXIT_STOP_DAISY = 9;
my $EXIT_START_DAISY = 10;
my $EXIT_PATCH_FAILED = 11;
my $EXIT_POSTPATCH = 12;


GetOptions(
	"help" => \$HELP,
	"version" => \$VERSION,
	"daisy" => \$DAISY,
	"daisydir:s" => \$DAISY_DIR,
	"edirupdate" => \$EDIRUPDATE,
	"convert-to-elavon" => \$CONVERT_TO_ELAVON,
	"install-ftd-files" => \$INSTALL_FTD_FILES,
	"debug" => \$DEBUG,
) || die "Error: invalid command line option, exiting...\n";


if ($CONVERT_TO_ELAVON) {
    $EDIRUPDATE = 0;
    $PATCH_VERSION = "ElavonConversion-1";

    @NU_BINARIES = qw(
	crdpos
	crdmenu
    );

    @NU_DATAFILES = qw(
	cutilscr.scr
	crd_scr.scr
	prsetscr.scr
    );
}
else {
    $PATCH_VERSION = "FMA2012FD";

    @NU_BINARIES = qw(
	edir
    );

    @NU_DATAFILES = qw(
    );
}


# --version
if ($VERSION) {
	print("$PROGNAME $CVS_REVISION\n");
	exit(0);
}

# --help
if ($HELP) {
	print("$PROGNAME $CVS_REVISION\n");
	usage();
	exit(0);
}


unless (defined($ARGV[0])) {
	print("ARGV[0] not defined... expecting path to daisy db dir... exiting\n");
	exit(1);
}
if ($ARGV[0] eq "") {
	print("ARGV[0] null... expecting path to daisy db dir... exiting\n");
	exit(1);
}

#
# If daisy not specificed on the command line, then verify the existence
# of the default daisy database.
#
if ($DAISY == 0) {
	unless (-d $DAISY_DIR) {
		print("Directory $DAISY_DIR not found... exiting.");
		exit(2);
	}
	$DAISY = 1;
}

$POSDIR = $DAISY_DIR;

# now we can specify the location of the logfile
my $LOGFILE = "$POSDIR/$LOGFILENAME";

unless (defined($ARGV[1])) {
	print("ARGV[1] not defined... expecting time stamp of patch... exiting\n");
	exit(1);
}
if ($ARGV[1] eq "") {
	print("ARGV[1] null... expecting time stamp of patch... exiting\n");
	exit(1);
}

$TIMESTAMP = $ARGV[1];


#
# -->
# --> Read the config file
# -->
#
read_configfile("$DAISY_DIR/config/$APPLYPATCH_CONFIG_FILE");


sub main
{
    my $exit_status = 0;

    loginfo("----------------------------------------------------------------");
    loginfo("Begin post patch for $PATCH_VERSION");
    loginfo("----------------------------------------------------------------");

    if ($CONVERT_TO_ELAVON) {
	my $daisy_dir = $ARGV[0];
	if ($ALTROOT ne "") {
	    $daisy_dir = $ALTROOT . $daisy_dir;
	}
	$exit_status = convert_to_elavon($daisy_dir);
    }

    if ($EDIRUPDATE) {
	$exit_status = postpatch_edir_update($ARGV[0]);
    }


    #
    # Remove the postpatch.pl script we are currently running.
    #
    remove_postpatch_script($ARGV[0]);

    #
    # Remove the file of Daisy FTD subs if present
    #
    remove_daisy_ftd_subs_file($ARGV[0]);

    loginfo("----------------------------------------------------------------");
    loginfo("Completed post patch for $PATCH_VERSION");
    loginfo("----------------------------------------------------------------");

    return($exit_status);
}


#######
#### Functions
#######


sub usage
{
    print("Usage:\n");
    print("$PROGNAME --version\n");
    print("$PROGNAME --help\n");
    print("$PROGNAME --daisy\n");
    print("$PROGNAME --daisydir=path\n");
    print("$PROGNAME --edirupdate\n");
    print("$PROGNAME --convert-to-elavon\n");
    print("\n");
}


#
# Read the config file located in default daisy db dir: /d/daisy/config
#
# Returns
#   0 on success
#   1 on error
#
sub read_configfile
{
    my $config_file = $_[0];

    unless ($config_file) {
	return(0);
    }
    if ($config_file eq "") {
	return(0);
    }
    unless (-e $config_file) {
	loginfo("Config File does not exist: $config_file");
	return(0);
    }

    loginfo("Reading config file: $config_file");

    unless (open(CONFIG, '<', $config_file)) {
	logerror("Can't open config file: $config_file");
	return(1);
    }

    while(<CONFIG>) {

	# qa-shopcode=nnnnnn00
	# Multiple instances of this line are allowed.
	if (/^(\s*)(qa-shopcode)(\s*)(=)(\s*)([[:print:]]+)$/i) {
		push(@CONFIG_QA_SHOPCODE, $6);
	}
    }

    close(CONFIG);

    return(0);
}


sub remove_postpatch_script
{
	my $postpatch_script_path = "$_[0]/bin/postpatch.pl";
	my $rc = 0;

	loginfo("Removing postpatch script: $postpatch_script_path");
	$rc = mysystem("rm -f $postpatch_script_path");
	if ($rc != 0) {
		logerror("Remove of postpatch script returns non-zero exit status: $rc");
	}

	return($rc);
}


sub remove_daisy_ftd_subs_file
{
    my $daisy_ftd_subs_path = "$_[0]/$DAISY_FTD_SUBS_FILE";
    my $rc = 0;

    if (-f $daisy_ftd_subs_path) {
	loginfo("Removing Daisy FTD subs file: $daisy_ftd_subs_path");
	$rc = mysystem("rm -f $daisy_ftd_subs_path");
	if ($rc != 0) {
	    logerror("Can't remove Daisy FTD subs file: $daisy_ftd_subs_path");
	}
    }

    return($rc);
}


#
# Get the system run level.
#
# Returns 0-6 on success, -1 on error
#
sub get_runlevel
{
    my $runlevel = -1;

    #
    # Output of runlevel is:
    #	(digit|letter) space digit newline
    # where the right most digit is the current runlevel
    #
    my $runlevelcmd = '/sbin/runlevel';
    if (open(RL, "$runlevelcmd |")) {
	while (<RL>) {
	    $runlevel = $_;
	    chomp($runlevel);
	    $runlevel =~ s/^. (\d)$/$1/;
	}
	close(RL);
    }
    else {
	showerror("Unable to use the run level command: $runlevelcmd");
    }

    return($runlevel);
}


#
# Set the system run level to value between 0 and 6.
#
# Returns 0 on success, -1 on error.
#
sub set_runlevel
{
    my ($new_runlevel) = @_;
    my $rc = 0;

    if ($new_runlevel < 0 || $new_runlevel > 6) {
	showerror("Can't happen: new system runlevel: $new_runlevel");
	return(-1);
    }

    system("/sbin/telinit $new_runlevel");
    if ($? != 0) {
	$rc = -1;
    }

    # The "init" man page mentions that "init" waits 5 seconds
    # between each of two kills, and testing reveals we need
    # to wait a bit for the runlevel to change.
    sleep(30);

    return($rc);
}


#
# Stop all Daisy processes.
#
# Return 0 on success, -1 on error
#
sub stop_daisy
{
        loginfo("Changing to system runlevel 4");
	# try to change the system runlevel
        unless (set_runlevel(4) == 0) {
            logerror("Can't change to system runlevel 4");
            return(-1);
        }

        # verify system runlevel is what we expect
        unless (get_runlevel() == 4) {
            logerror("Can't verify system runlevel is 4");
            return(-1);
        }
        loginfo("System runlevel changed to 4");

	# now kill off all daisy processes
	my $killemall_cmd = "/d/daisy/bin/killemall.pl";
        if (-f $killemall_cmd) {
            loginfo("Killing all Daisy processes");
            system("$killemall_cmd");
	    if ($? != 0) {
                showerror("Unable to kill all Daisy processes");
                return(-1);
            }
            loginfo("All Daisy processes killed");
        }
	else {
	    logerror("Expecting script to exist: $killemall_cmd");
	    return(-1);
	}

	return(0);
}


#
# Start Daisy.
#
# Returns 0 on success, -1 on error
#
sub start_daisy()
{
    my $rc = 0;

    if (set_runlevel(3) != 0) {
	logerror("Can't set runlevel to 3");
	return(-1);
    }

    if (get_runlevel() != 3) {
	logerror("Unable to return to runlevel 3");
	return(-1);
    }

    loginfo("Daisy restarted and system back to run level 3");

    return(0);
}


#
# Read shop code from Dove control file in default daisy database dir.
#
# Returns:
#   shop code as 8 digit string if found else empty string
#
sub get_shopcode
{
    my $daisy_db_dir = $_[0];
    my $shopcode = "";

    if (-d $daisy_db_dir) {
	my $dovectrl_file = "$daisy_db_dir/dovectrl.pos";

	unless (-f $dovectrl_file) {
	    logerror("Dove control file not found: $dovectrl_file");
	    return "";
	}

	unless (open(DF, $dovectrl_file)) {
	    logerror("Can't open Dove control file: $dovectrl_file");
	    return "";
	}

	my $buffer = "";
	my $bufsize = 50;
	my $rc = sysread(DF, $buffer, $bufsize);

	unless (defined($rc)) {
	    logerror("Error reading Dove control file: $!\n");
	    return "";
	}
	if ($rc == 0) {
	    logerror("Could not read Dove control file: $dovectrl_file");
	    return "";
	}
	if ($rc != $bufsize) {
	    logerror("Short read from Dove control file: $dovectrl_file");
	    logerror("Expected read of $bufsize but got $rc");
	    return "";
	}

	close(DF);

	$shopcode = substr($buffer, 30, 8);
    }
}


#
# Look up the shop code from "the list", ie list of shops that have purchased
# the FTD files.
#
# Returns:
#   1 if shop code is on the list, else 0
#
sub lookup_shopcode
{
    my $daisy_db_dir = $_[0];
    my $shopcode = $_[1];
    my $rc = 0;
    my $daisy_ftd_subs_path = "$daisy_db_dir/$DAISY_FTD_SUBS_FILE";

    #
    # Check to see if the shopcode is one of the QA shopcodes that
    # may be specified in the config file. 
    #
    if (@CONFIG_QA_SHOPCODE) {
	foreach (@CONFIG_QA_SHOPCODE) {
	    if (/$shopcode/) {
		loginfo("Shopcode matched QA shopcode from config file: $shopcode");
		loginfo("Allowing FTD subscription");
		return(1);
	    }
	}
    }

    unless (-f $daisy_ftd_subs_path) {
	logerror("Daisy FTD subscribers file not found: $daisy_ftd_subs_path");
	loginfo("Allowing FTD subscription");
	return(1);
    } 

    unless (open(SF, '<', $daisy_ftd_subs_path)) {
	logerror("Can't open Daisy FTD subscribers file: $daisy_ftd_subs_path");
	loginfo("Allowing FTD subscription");
	return(1);
    }

    while (<SF>) {

        # skip comments or blank lines
        next if ( (/^(\s*)#/) || (/^(\s*)$/) );

	# remove newline
	chomp;

	# remove leading and trailing SPACE chars
	my $subscriber = trim($_);

	# finally do the comparison
	if ($subscriber eq $shopcode) {
	    loginfo("Shopcode matched shopcode from subscriber file: $shopcode");
	    loginfo("Allowing FTD subscription");
	    $rc = 1;
	    last;
	}
    }

    close(SF);

    return($rc);
}


#
# Find the shop code, and lookup the shop code in the "paid" list;
# if the shop code is not on the paid list, then remove the FTD files
# from the backup (if they exist) and from the current daisy db dir.
#
# Returns
#   0 on success, non-0 on error
#
sub resolve_ftd_files
{
    my $daisy_db_dir = $_[0];
    my $rc = 0;

    my $DEF_DAISY_DB_DIR = "/d/daisy";

    #
    # If the command line option specified, leave the FTD files
    # installed without any further checks.
    #
    if ($INSTALL_FTD_FILES) {
	loginfo("FTD files installation forced with \"--install-ftd-files\" option");
	return(0);
    }

    my $shopcode = get_shopcode($DEF_DAISY_DB_DIR);
    if ($shopcode eq "") {
	logerror("Could not get shop code");
	return(1);
    }

    #
    # At this point, there may be old FTD files in the backup dir and
    # there may be new FTD files in the daisy db dir.
    #
    # Now, the shopcode must be on "the list" or both any old FTD files and
    # any new FTD files are removed.
    #
    # If the shopcode is not on "the list", then remove the new FTD files from
    # the daisy db dir and the old FTD files from the backup dir.

    unless (lookup_shopcode($daisy_db_dir, $shopcode)) {

	loginfo("Shop with shop code $shopcode is a non-FTD subscriber");
	loginfo("Removing new FTD files");

	$rc = mysystem("rm -f $daisy_db_dir/*.ftd");
	if ($rc != 0) {
	    logerror("Removal of new FTD files failed ($rc)");
	}

	loginfo("Removing old FTD files");

	my @ftd_backup_files = glob("$daisy_db_dir/backups/*.ftd.prepatch.*");
	foreach (@ftd_backup_files) {
	    $rc = mysystem("rm -f $_");
	}
    }

    return($rc);
}


sub resolve_hsp_files
{
	my $daisy_db_dir = $_[0];
	my $rc = 0;

	my @hsp_backup_files = glob("$daisy_db_dir/backups/hnot*.hsp.prepatch.*");

	unless (@hsp_backup_files) {
		return($rc);
	}

	foreach my $hsp_backup_file (@hsp_backup_files) {

		# isolate the filename
		my $hsp_filename = basename($hsp_backup_file);

		# convert it back to original
		if ($hsp_filename =~ /(.*)\.prepatch\..*$/) {
			$hsp_filename = $1;
		}

		loginfo("Restoring pervious hospital note file: $hsp_filename");
		$rc = mysystem("cp $hsp_backup_file $daisy_db_dir/$hsp_filename");
		if ($rc != 0) {
			logerror("Restoration of hospital note file ($hsp_filename) failed ($rc)");
			last;
		}
	}

	return($rc);
}


#
# resolve_binaries
#
# foreach new binary file
#	form a platform specific file name
#	if there is platform specific file
#		copy the original binary to the backup directory
#		mv the platform specific file to the original file
#
#
# Return 0 on success, non-zero otherwise
#
sub resolve_binaries
{
    my $daisy_dir = $_[0];
    my $rc = 0;

    determine_os();

    #
    # "RHWS5" is really "Redhat Enterprise Linux Client" and for the
    # purposes of chosing the correct binary, we must treat "Client"
    # systems like "Server" systems.
    #
    if ($OS eq "RHWS5") {
	$OS = "RHEL5";
    }

    if ($OS ne "RH72" && $OS ne "FC3" && $OS ne "FC5" && $OS ne "RHEL5") {
	logerror("Unsupported platform: $OS");
	return(1);
    }

    #
    # FIXME: remove non-applicable platform binaries 
    #

    foreach my $nu_binary (@NU_BINARIES) {

	my $platform_binary = $nu_binary . "-" . $OS;

	if (-f "$daisy_dir/$platform_binary") {
	    loginfo("Installing new binary: $nu_binary for platform: $OS");
	    my $tmp_rc = mysystem("cp $daisy_dir/$nu_binary $daisy_dir/backups");
	    if ($tmp_rc != 0) {
		logerror("Backup of $nu_binary failed ($rc)");
		$rc = 1;
	    }
	    else {
		$tmp_rc = mysystem("mv $daisy_dir/$platform_binary $daisy_dir/$nu_binary");
		if ($tmp_rc != 0) {
		    logerror("Installation of $nu_binary failed ($rc)");
		    $rc = 1;
		}

		system("chown tfsupport:daisy $daisy_dir/$nu_binary");
		system("chmod 550 $daisy_dir/$nu_binary");
	    }
	}
	else {
	    logerror("New binary for platform ($OS) not available in patch: $nu_binary");
	    $rc = 1;
	}
    }

    return($rc);
}		


sub resolve_data_files
{
    my $daisy_db_dir = $_[0];
    my $rc = 0;

    # nothing to do

    return($rc);
}


#
# (1) If there are no FTD files in the backup dir, then the assumption is
#     that this daisy system does not subscribe to FTD.  So, remove
#     the FTD files from the current daisy db dir.
#
# (2) If there are "hospital note files" in the backup dir, that means
#     there were some in the daisy db dir before the patch was applied...
#     so the new empty files need to be removed from the current daisy
#     db dir and the old ones copied from the backup dir to the current
#     daisy db dir.
#
# (3) If there were new or updated binaries, remove the old ones, and
#     install the new ones.
#
sub postpatch_edir_update
{
	my $daisy_db_dir = $_[0];
	my $rc = 0;

	if (resolve_ftd_files($daisy_db_dir)) {
		$rc++;
	}

	if (resolve_hsp_files($daisy_db_dir)) {
		$rc++;
	}

	if (resolve_binaries($daisy_db_dir) != 0) {
		$rc++;
	}

	return($rc);
}


# trim whitespace from start and end of string
sub trim
{
	my $string = $_[0];

	$string =~ s/^\s+//;
	$string =~ s/\s+$//;

	return $string;
}


sub fill_left
{
    my $string = $_[0];
    my $fill_char = $_[1];
    my $str_max = $_[2];

    my $str_len = length($string);

    if ($str_len < $str_max) {
	my $fill_size = $str_max - $str_len;
	for (my $i = 0; $i < $fill_size; $i++) {
	    $string = $fill_char . $string;
	}
    }

    return($string);
}


sub backout_patch
{
    my $daisy_dir = $_[0];

    my $os = determine_os();

    foreach my $thisfile (@NU_BINARIES) {

	# since this is a binary, when the file was backed up,
	# a platform suffix was added to it so we must add the
	# same suffix to the name.
	my $platform_binary = $thisfile . "-" . $os;

	my $src_file = "$daisy_dir/backups/$platform_binary.prepatch.$TIMESTAMP";
	my $dst_file = "$daisy_dir/$thisfile";

	if (! -f $src_file) {
	    loginfo("Warning: can't back out patched file: $src_file does not exist");
	    next;
	}

	loginfo("Backing out $src_file -> $dst_file");
	mysystem("cp -f $src_file $dst_file 2> /dev/null");
	if ($? != 0) {
	    # We're pretty much hosed at this point.
	    logerror("Backout of $src_file $dst_file Failed ($?)");
	}
    }

    foreach my $thisfile (@NU_DATAFILES) {

	my $src_file = "$daisy_dir/backups/$thisfile.prepatch.$TIMESTAMP";
	my $dst_file = "$daisy_dir/$thisfile";

	if (! -f $src_file) {
	    loginfo("Warning: can't back out patched file: $src_file does not exist");
	    next;
	}

	loginfo("Backing out $src_file -> $dst_file");
	mysystem("cp -f $src_file $dst_file 2> /dev/null");
	if ($? != 0) {
	    # We're pretty much hosed at this point.
	    logerror("Backout of $src_file $dst_file Failed ($?)");
	}
    }

    # fix up the perms after copying files back
    my $perms_cmd = "$daisy_dir/bin/dsyperms.pl";
    if (-e $perms_cmd) {
	loginfo("After backing out patch, setting perms on: $daisy_dir");
	mysystem("$perms_cmd $daisy_dir");
	if ($? != 0) {
	    logerror("Error occurred running script: $perms_cmd");
	}
    }
    else {
	logerror("Should not happen: script does not exist: $perms_cmd");
    }
}


#
# Determines if settelment record count is zero
#
# Returns 1 if true else 0
#
sub is_settle_rec_count_zero
{
    my $settle_file_path = $_[0];
    my $buffer = "";
    my $bufsize = 2; # first two bytes
    my $rc = 0; # assume false

    if (open(SF, $settle_file_path)) {
	my $readsize = sysread(SF, $buffer, $bufsize);
	if (defined($readsize) && $readsize == $bufsize) {
	    my $byte = ord(substr($buffer, 0, 1));
	    if ($byte == 0) {
		$byte = ord(substr($buffer, 1, 1));
		if ($byte == 0) {
		    $rc = 1;
		}
	    }
	}
	else {
	    logerror("Error reading settle file: $settle_file_path\n");
	    logerror("Error: $!\n");
	}
	close(SF);
    }
    else {
	logerror("Open of settle file failed: $settle_file_path");
	logerror("Error: $!\n");
    }

    return($rc);
}


#
# Convert NDC2 terminal id to Elavon terminal id
#
# if (the bank id is "203600") {
#   then it's a USA shop configured for Global, so do the following:
#   a)  if the terminal ID does not start with "167", and
#       it is less than or equal to 13 characters in length,
#       prefix string "167" to the front of the terminal ID.
#   b)  add ascii zeroes to the front of the terminal ID, as needed,
#       to make it have length 16
# }
#
# elsif (the bank id is "039100") {
#    then it's a Canadian shop configured for Global, so do the following:
#    a)  DO NOT add the prefix "167", but add ascii zeroes to the front
#        of the terminal ID, as needed, to make it have length 16
# }
#
# else {
#   treat the shop as it is NOT to be processed on Elavon,
#   as the bank ID is clearly not Telefloras.
#   a)  assign the termainal id 16 SPACEs
# }
#
sub convert_terminal_id
{
    my $bank_id = $_[0];
    my $terminal_id = $_[1];

    my $terminal_id_size = length($terminal_id);

    # USA Global East
    if ($bank_id eq "203600") {
	if ($terminal_id_size <= 13) {
	    if ($terminal_id !~ /^167/) {
		$terminal_id = "167" . $terminal_id;
	    }
	}
	if ($terminal_id_size < 16) {
	    $terminal_id = fill_left($terminal_id, "0", 16);
	}
    }

    # Canadian Global East
    elsif ($bank_id eq "039100") {
	if ($terminal_id_size < 16) {
	    $terminal_id = fill_left($terminal_id, "0", 16);
	}
    }

    # Unknown payment processor
    else {
	$terminal_id = "";
	$terminal_id = fill_left($terminal_id, " ", 16);
    }

    return($terminal_id);
}


#
# Get a field from a control file
#
# Input
#   arg[0]: full path to the control file
#   arg[1]: buffer size
#   arg[2]: index of field begin
#   arg[3]: field length
# 
# Return
#   field on success, empty string on error
#
sub get_control_file_field
{
    my $ctrl_file = $_[0];
    my $bufsize = $_[1];
    my $field_begin = $_[2];
    my $field_size = $_[3];
    my $field = "";
    my $buffer = "";

    if (open(DF, '<', "$ctrl_file")) {
	my $rc = sysread(DF, $buffer, $bufsize);
	if (defined($rc) && $rc == $bufsize) {
	    $field = substr($buffer, $field_begin, $field_size);
	}
	else {
	    logerror("Error reading control file: $ctrl_file");
	    logerror("Error: $!");
	}
	close(DF);
    }
    else {
	logerror("Error opening control file: $ctrl_file");
	logerror("Error: $!");
    }

    return($field);
}


#
# The layout of the Global East control file "ndc2ctrl.pos":
#
# 124 bytes long
#
# bank id: 6 bytes of ascii
# filler: 2 zero bytes
# terminal id: 15 bytes of ascii
# filler: 1 zero bytes
# remainder: 100 zero bytes # best set to zero, but might contain anything.
#
# Example of "real" ndc2ctrl.pos file, taken from a production daisy system:
# $ od -c ndc2ctrl.pos
# 0000000   2   0   3   6   0   0  \0  \0   0   2   9   7   1   5   6
# 0000020                              \0  \0  \0  \0  \0  \0  \0  \0  \0
# 0000040  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0
# *
# 0000160  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0  \0
# 0000174
#
#
# The layout of the Elavon control file "elavctrl.pos":
#
# 124 bytes long
#
# bank id: 6 bytes of ascii
# filler: 2 zero bytes
# terminal id: 16 bytes of ascii
# filler: 2 zero bytes
# remainder: 98 zero bytes # best set to zero, but might contain anything.
#
# Input
#   First arg is a full path to the Elavon control file
#   Second arg is bank id
#   Third arg is terminal id
#
# Return
#   0 on success, non-zero on error
#
sub generate_elavon_ctrl_file
{
    my $elavon_ctrl_file = $_[0];
    my $bank_id = $_[1];
    my $terminal_id = $_[2];

    my $df;
    my $rc;

    unless (open($df, '>', $elavon_ctrl_file)) {
	logerror("Open of new Elavon control file failed: $elavon_ctrl_file");
	return(1);
    }

    my $filler = 0;

    $rc = syswrite($df, $bank_id, 6);
    $filler = pack('x2', 0);
    $rc = syswrite($df, $filler, 2);
    $rc = syswrite($df, $terminal_id, 16);
    $filler = pack('x100', 0);
    $rc = syswrite($df, $filler, 100);

    unless (defined($rc)) {
	logerror("Error writing new Elavon control file: $!\n");
	return(1);
    }

    if ($rc == 0) {
	logerror("Error writing new Elavon control file returns EOF: $elavon_ctrl_file\n");
	return(1);
    }

    close($df);

    return(0);
}


#
# Make an Elavon control file based on a Global control file
#
# Input
#   First arg is a full path to a either a daisy database directory or
#   a credit card processing directory within a daisy database directory
# 
# Return
#   0 --> success
#   1 --> Elavon control file already exists
#   2 --> NDC2 control file does not exist
#   3 --> NDC2 control file wrong length
#   4 --> Can't get bank id or terminal id from NDC2 control file
#   5 --> Can't generate new Elavon control file
#
sub make_elavon_ctrl_file
{
    my $daisy_dir = $_[0];
    my $elavon_ctrl_file = "$daisy_dir/elavctrl.pos";
    if ($ALTROOT ne "") {
	$elavon_ctrl_file = $ALTROOT . $elavon_ctrl_file;
    }

    if (-e $elavon_ctrl_file) {
	logerror("Elavon control file already exists: $elavon_ctrl_file");
	return(1);
    }

    my $ndc2_ctrl_file = "$daisy_dir/ndc2ctrl.pos";
    if ($ALTROOT ne "") {
	$ndc2_ctrl_file = $ALTROOT . $ndc2_ctrl_file;
    }

    unless (-e $ndc2_ctrl_file) {
	logerror("NDC2 control file does not exist: $ndc2_ctrl_file");
	return(2);
    }

    my $ndc2_ctrl_file_size = -s $ndc2_ctrl_file;

    if ($ndc2_ctrl_file_size != 124) {
	logerror("NDC2 control file should be 124 bytes in length: $ndc2_ctrl_file");
	return(3);
    }

    my $bank_id = get_control_file_field($ndc2_ctrl_file, 124, 0, 6);
    my $terminal_id = get_control_file_field($ndc2_ctrl_file, 124, 8, 15);

    if ($bank_id eq "" || $terminal_id eq "") {
	logerror("Can't get bank id or terminal id from NDC2 control file: $ndc2_ctrl_file");
	return(4);
    }

    $bank_id = trim($bank_id);
    $terminal_id = trim($terminal_id);

    $terminal_id = convert_terminal_id($bank_id, $terminal_id);

    if (generate_elavon_ctrl_file($elavon_ctrl_file, $bank_id, $terminal_id) != 0) {
	logerror("Can't generate new Elavon control file: $elavon_ctrl_file");
	return(5);
    }

    unless (-e $elavon_ctrl_file) {
	logerror("Can't happen: Elavon control file does not exist: $elavon_ctrl_file");
	return(6);
    }

    system("chown tfsupport:daisy $elavon_ctrl_file");
    if ($? != 0) {
	logerror("Can't chown Elavon control file: $elavon_ctrl_file");
	return(7);
    }

    system("chmod 660 $elavon_ctrl_file");
    if ($? != 0) {
	logerror("Can't chmod Elavon control file: $elavon_ctrl_file");
	return(8);
    }

    loginfo("New elavon ctrl file generated: $elavon_ctrl_file");

    return(0);
}


#
# Convert a card control file for a daisy dir.
#
# Return 0 on success, non-zero otherwise
#
sub convert_card_ctrl_file
{
    my $daisy_dir = $_[0];

    my $card_ctrl_file = "$daisy_dir/crdctrl.pos";
    if ($ALTROOT ne "") {
	$card_ctrl_file = $ALTROOT . $card_ctrl_file;
    }

    my $df;
    my $rc;

    if (open($df, '+<', $card_ctrl_file)) {
	seek($df, 0, 0);
	my $byte = pack('C', 5);
	$rc = syswrite($df, $byte, 1);
	$byte = pack('x', 0);
	$rc = syswrite($df, $byte, 1);
    }
    close($df);

    system("chown tfsupport:daisy $card_ctrl_file");
    system("chmod 660 $card_ctrl_file");

    loginfo("Converted card ctrl file: $card_ctrl_file");

    return(0);
}


#
# Generate an empty settle file for a daisy dir.
# Per Chris Moth, 20111216.
#
# Return 0 on success, non-zero otherwise
#
sub make_settle_files
{
    my $daisy_dir = $_[0];
    my $settle_file = "$daisy_dir/settfile.pos";

    if ($ALTROOT ne "") {
	$settle_file = $ALTROOT . $settle_file;
    }

    if (-e $settle_file) {
	my $settle_file_old = $settle_file . ".old";
	if ($ALTROOT ne "") {
	    $settle_file_old = $ALTROOT . $settle_file_old;
	}

	mysystem("mv $settle_file $settle_file_old");
    }

    mysystem("dd if=/dev/zero of=$settle_file bs=1024 count=8 > /dev/null 2> /dev/null ");

    unless (-e $settle_file) {
	logerror("Can't generate empty settle file: $settle_file");
	return(1);
    }

    system("chown tfsupport:daisy $settle_file");
    system("chmod 660 $settle_file");

    loginfo("New settle file generated: $settle_file");

    #
    # make the numbered settle files too per Chris Moth, 20111216.
    #
    my @numbered_settle_files = qw(
	$daisy_dir/settfil0.pos
	$daisy_dir/settfil1.pos
	$daisy_dir/settfil2.pos
    );

    foreach (@numbered_settle_files) {
	mysystem("cp $settle_file $_");
	mysystem("chown tfsupport:daisy $_");
	mysystem("chmod 660 $_");
    }

    return(0);
}


#
# Generate an empty card input file for a daisy dir.
#
# Return 0 on success, non-zero otherwise
#
sub make_card_input_file
{
    my $daisy_dir = $_[0];
    my $card_input_file = "$daisy_dir/crdinput.pos";

    if ($ALTROOT ne "") {
	$card_input_file = $ALTROOT . $card_input_file;
    }

    if (-e $card_input_file) {
	my $card_input_file_old = "$daisy_dir/crdinput.pos.old";
	if ($ALTROOT ne "") {
	    $card_input_file_old = $ALTROOT . $card_input_file_old;
	}

	mysystem("mv $card_input_file $card_input_file_old");
    }

    mysystem("dd if=/dev/zero of=$card_input_file bs=1024 count=8 > /dev/null 2> /dev/null ");

    unless (-e $card_input_file) {
	logerror("Can't generate empty card input file: $card_input_file");
	return(1);
    }

    system("chown tfsupport:daisy $card_input_file");
    system("chmod 660 $card_input_file");

    loginfo("New card input file generated: $card_input_file");

    return(0);
}


#
# Generate a new card inet file for a daisy dir.
#
# Return 0 on success, non-zero otherwise
#
sub make_card_inet_file
{
    my $daisy_dir = $_[0];
    my $card_inet_file = "$daisy_dir/crdinet.pos";

    if ($ALTROOT ne "") {
	$card_inet_file = $ALTROOT . $card_inet_file;
    }

    if (-e $card_inet_file) {
	my $card_inet_file_old = "$daisy_dir/crdinet.pos.old";
	if ($ALTROOT ne "") {
	    $card_inet_file_old = $ALTROOT . $card_inet_file_old;
	}

	mysystem("mv $card_inet_file $card_inet_file_old");
    }

    my $max_url_size = 199;
    my $max_file_size = 1404;

    my $df;
    my $rc;

    unless (open($df, '>', $card_inet_file)) {
	logerror("Open of new card inet file failed: $card_inet_file");
	return(1);
    }

    # first, 2 x 2 byte fields
    my $byte_val = pack('C', 1);
    $rc = syswrite($df, $byte_val, 1);
    $byte_val = pack('x', 0);
    $rc = syswrite($df, $byte_val, 1);
    $byte_val = pack('C', 1);
    $rc = syswrite($df, $byte_val, 1);
    $byte_val = pack('x', 0);
    $rc = syswrite($df, $byte_val, 1);

    # next, first url
    my $url_field = "https://webgate.viaconex.com";
    my $url_field_size = length($url_field);
    if ($url_field_size > $max_url_size) {
	logerror("URL for field 1 of $card_inet_file must be < 200 chars: $url_field");
	return(1);
    }

    my $padded_url_field = sprintf("%-${max_url_size}s", $url_field);
    $rc = syswrite($df, $padded_url_field, $max_url_size);

    # then zero byte terminator
    $byte_val = pack('x', 0);
    $rc = syswrite($df, $byte_val, 1);

    # next, second url
    $url_field = "";
    $padded_url_field = sprintf("%-${max_url_size}s", $url_field);
    $rc = syswrite($df, $padded_url_field, $max_url_size);

    # then zero byte terminator
    $byte_val = pack('x', 0);
    $rc = syswrite($df, $byte_val, 1);

    # finally NUL file to file length of 1404
    # 1404 - (2 + 2 + 200 + 200)
    # 1404 - (404)
    # 1000
    $byte_val = pack('x1000', 0);
    $rc = syswrite($df, $byte_val, 1000);

    close($df);

    if ($rc == 0) {
	logerror("Error writing new card inet file: $card_inet_file\n");
	return(1);
    }

    my $card_inet_file_size = -s $card_inet_file;
    if ($card_inet_file_size != $max_file_size) {
	logerror("Generated card inet file size incorrect: $card_inet_file\n");
	loginfo("Expecting card inet file size to be: $max_file_size\n");
	return(1);
    }

    system("chown tfsupport:daisy $card_inet_file");
    system("chmod 660 $card_inet_file");

    loginfo("New card inet file generated: $card_inet_file");

    return(0);
}


sub digits_of_phone_field
{
    my $field_old_val = $_[0];
    my $phone_digits;

    for (my $i=0; $i < length($field_old_val); $i++) {
	my $next_char = substr($field_old_val, $i, 1);
	if ($next_char =~ /\d/) {
	    $phone_digits .= $next_char;
	}
    }

    return($phone_digits);
}


sub read_phone_field
{
    my $df = $_[0];
    my $field_start = $_[1];
    my $field_size = $_[2];
    my $file_path = $_[3];

    my $field_buf = "";
    my $phone_field = "";

    seek($df, $field_start, 0);

    my $rc = sysread($df, $field_buf, $field_size);
    if (defined($rc)) {
	if ($rc == 0) {
	    logerror("Unexpected read of end of file: $file_path");
	}
	elsif ($rc != $field_size) {
	    logerror("Misread of card modem file: $file_path");
	}
	else {
	    $phone_field = $field_buf;
	}
    }
    else {
	logerror("Error reading card modem file: $file_path");
	logerror("Error: $!\n");
    }

    return($phone_field);
}


#
# Extract all the digits from the phone field and
# if the first digit is a "9", then prefix the new number
# with "9," else leave out any prefix.
#
sub convert_phone_field
{
    my $field_old_val = $_[0];
    my $field_nu_val = $_[1];

    my $converted_field = "";

    my $phone_digits = digits_of_phone_field($field_old_val);
    if (substr($phone_digits, 0, 1) =~ /9/) {
	$converted_field = "9,";
    }
    $converted_field .= $field_nu_val;

    return($converted_field);
}


sub write_phone_field
{
    my $df = $_[0];
    my $field_val = $_[1];
    my $field_start = $_[2];
    my $field_size = $_[3];

    seek($df, $field_start, 0);
    my $bytes_written = syswrite($df, $field_val, $field_size);
    my $byte = pack('x', 0);
    $bytes_written += syswrite($df, $byte, 1);

    return($bytes_written);
}


#
# Primary in phone number field 1
# Primary in phone number field 3
# Secondary in phone number field 2
# Secondary in phone number field 4
#
# Primary   == (800) 741-3737
# Secondary == (800) 972-4608  
#
# phone number fields are 25 chars, NUL terminated
# phone numbers are right filled with SPACEs so they
# are always 24 bytes in length.
#
# File bytes are numbered starting from 0
#
# phone number field 1 starts at byte 29
# phone number field 2 starts at byte 151
# phone number field 3 starts at byte 273
# phone number field 4 starts at byte 395
#
#
# Return 0 on success, non-zero otherwise
#
sub convert_card_modem_file
{
    my $daisy_dir = $_[0];

    my $file_path = "$daisy_dir/crdmodem.pos";
    if ($ALTROOT ne "") {
        $file_path = $ALTROOT . $file_path;
    }

    my $primary_phone   = "18007413737";
    my $secondary_phone = "18009724608";
    my $blank_phone     = "                         ";

    my @phone_records = (
	[  28, 25, $primary_phone ] ,
	[ 150, 25, $secondary_phone ],
	[ 272, 25, $primary_phone ],
	[ 394, 25, $secondary_phone ],
    );

    my $df;
    my $rc = 0;
    my $loop_counter = 1;

    if (open($df, '+<', $file_path)) {

	foreach my $record (@phone_records) {

	    my $field_start  = ${$record}[0];
	    my $field_size   = ${$record}[1];
	    my $field_nu_val = ${$record}[2];

	    my $field_old_val = read_phone_field($df, $field_start, $field_size, $file_path);
	    if ($DEBUG) {
		loginfo("crdmodem.pos: record $loop_counter: old value: <$field_old_val>");
	    }

	    #
	    # if the phone field is either an empty string or all SPACEs,
	    # then just leave it alone.
	    #
	    if ($field_old_val eq "" || $field_old_val eq $blank_phone) {
		loginfo("Skipping blank phone field: file $file_path, record $loop_counter");
		$loop_counter++;
		next;
	    }

	    my $field_converted = convert_phone_field($field_old_val, $field_nu_val);
	    if ($DEBUG) {
		loginfo("crdmodem.pos: record $loop_counter: converted value: <$field_converted>");
	    }
	
	    my $field_padded = sprintf("%-${field_size}s", $field_converted);
	    if ($DEBUG) {
		loginfo("crdmodem.pos: record $loop_counter: padded value: <$field_padded>");
	    }

	    my $bytes_written = write_phone_field($df, $field_padded, $field_start, $field_size);
	    if ($DEBUG) {
		loginfo("crdmodem.pos: record $loop_counter: bytes written: <$bytes_written>");
	    }
	    if ($bytes_written != $field_size + 1) {
		$rc = $loop_counter;    # indicate which record had the issue
		last;
	    }

	    $loop_counter++;
	}
    }
    else {
	logerror("Can't open card modem file: $file_path");
	return(1);
    }

    close($df);

    system("chown tfsupport:daisy $file_path");
    system("chmod 660 $file_path");

    if ($rc != 0) {
	logerror("Error processing record $rc of card modem file: $file_path");
	return($rc);
    }

    loginfo("Converted card modem file: $file_path");

    return(0);
}


#
# Convert all the config files for a daisy dir.
#
# Return 0 on success, non-zero otherwise
#
sub convert_config_files
{
    my $daisy_dir = $_[0];

    if (make_elavon_ctrl_file($daisy_dir) != 0) {
	logerror("Can't generate Elavon control file for $daisy_dir");
	return(1);
    }

    if (make_settle_files($daisy_dir) != 0) {
	logerror("Can't generate empty settle file for $daisy_dir");
	return(1);
    }

    if (convert_card_ctrl_file($daisy_dir) != 0) {
	logerror("Can't convert card control file for $daisy_dir");
	return(1);
    }

    if (make_card_input_file($daisy_dir) != 0) {
	logerror("Can't generate empty card input file for $daisy_dir");
	return(1);
    }

    if (make_card_inet_file($daisy_dir) != 0) {
	logerror("Can't generate card inet file for $daisy_dir");
	return(1);
    }

    if (convert_card_modem_file($daisy_dir) != 0) {
	logerror("Can't convert card modem file for $daisy_dir");
	return(1);
    }

    return(0);
}


#
# Install all the files from the patch file into a daisy dir.
#
# Return 0 on success, non-zero otherwise
#
sub install_patch_files
{
    my $daisy_dir = $_[0];

    if (resolve_binaries($daisy_dir) == 0) {
	if (resolve_data_files($daisy_dir) == 0) {
	    return(0);
	}
	else {
	    logerror("Can't resolve data files from patch for $daisy_dir");
	}
    }
    else {
	logerror("Can't resolve binary files from patch for $daisy_dir");
    }

    return(1);
}


#
# If there were new or updated binaries, remove the old ones, and
#     install the new ones.
#
# Returns:
#   0 on success or conversion not needed, non-zero on error
#   1 --> Settlement records not zero, can't convert
#   2 --> Can't generate Elavon control file, can't convert
#
sub convert_to_elavon
{
    my $daisy_dir = $_[0];
    my $exit_status = $EXIT_OK;
    my $PRECHECK_OK = 1;
    my $PRECHECK_SKIP = 2;
    my $PRECHECK_ERROR = 3;
    my $precheck_status = $PRECHECK_OK;

    #
    # These checks should have already been done, but it doesn't hurt
    # to do them again and it allows postpatch.pl program to be run
    # alone when doing testing.
    #

    my $elavon_ctrl_file = "$daisy_dir/elavctrl.pos";
    my $settle_file = "$daisy_dir/settfile.pos";
    my $card_ctrl_file = "$daisy_dir/crdctrl.pos";
    my $ndc2_ctrl_file = "$daisy_dir/ndc2ctrl.pos";
    my $card_inet_file = "$daisy_dir/crdinet.pos";
    my $card_modem_file = "$daisy_dir/crdmodem.pos";

    if (-e $elavon_ctrl_file) {
	loginfo("An Elavon control file already exists: $elavon_ctrl_file");
	loginfo("Skipping directory with existing Elavon control: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    elsif (! -e $settle_file) {
	loginfo("Settle file does not exist: $settle_file");
	loginfo("Skipping directory with no settle file: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    elsif (! is_settle_rec_count_zero($settle_file)) {
	logerror("Non-zero record count in settle file: $settle_file");
	loginfo("Skipping directory with outstanding settlements: $daisy_dir");
	$precheck_status = $PRECHECK_ERROR;
    }

    elsif (! -e $card_ctrl_file) {
	loginfo("Card control file does not exist: $card_ctrl_file");
	loginfo("Skipping directory with no card control file: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    elsif (! -e $ndc2_ctrl_file) {
	loginfo("NDC2 control file does not exist: $ndc2_ctrl_file");
	loginfo("Skipping directory with no NDC2 control file: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    elsif (! -e $card_inet_file) {
	loginfo("Card inet file does not exist: $card_inet_file");
	loginfo("Skipping directory with no card inet file: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    elsif (! -e $card_modem_file) {
	loginfo("Card modem file does not exist: $card_modem_file");
	loginfo("Skipping directory with no card modem file: $daisy_dir");
	$precheck_status = $PRECHECK_SKIP;
    }

    if ($precheck_status != $PRECHECK_OK) {
	$exit_status = ($precheck_status == $PRECHECK_SKIP) ? $EXIT_OK : $EXIT_PATCH_FAILED;
	backout_patch($daisy_dir);
	return($exit_status);
    }

    #
    # At last, we can actually do the conversion.
    #

    $exit_status = $EXIT_PATCH_FAILED;

    loginfo("Stopping Daisy...");
    if (stop_daisy() != 0) {
	logerror("Could not stop Daisy");
	return($EXIT_STOP_DAISY);
    }
    loginfo("Daisy stopped");

    loginfo("Converting Daisy config files in: $daisy_dir");
    if (convert_config_files($daisy_dir) == 0) {
	loginfo("Daisy config files converted");

	loginfo("Installing patch files in: $daisy_dir");
	if (install_patch_files($daisy_dir) == 0) {
	    loginfo("Patch files installed");
	    $exit_status = $EXIT_OK;
	}
	else {
	    logerror("Can't install patch files");
	}
    }
    else {
	logerror("Can't convert Daisy config files");

    }

    if ($exit_status != $EXIT_OK) {
	system("rm $elavon_ctrl_file");
	backout_patch($daisy_dir);
    }

    loginfo("Starting Daisy...");
    if (start_daisy() != 0) {
	logerror("Could not start Daisy");
	return($EXIT_START_DAISY);
    }
    loginfo("Daisy restarted");

    return($exit_status);
}


sub determine_os
{
	$OS = "";

	# We could also use /usr/bin/lsb-release
	open(FILE, '<', "/etc/redhat-release");
	while(<FILE>) {

		# Fedora Core 3
		if(/(Fedora)(\s+)(Core)(\s+)(release)(\s+)(3)/) {
			$OS = "FC3";
			last;
		}

		# Fedora Core 4
		if(/(Fedora)(\s+)(Core)(\s+)(release)(\s+)(4)/) {
			$OS = "FC4";
			last;
		}

		# Fedora Core 5
		if(/(Fedora)(\s+)(Core)(\s+)(release)(\s+)(5)/) {
			$OS = "FC5";
			last;
		}

		# Redhat Enterprise Linux Client Workstation 5
		if( /(Client release)(\s+)(5)/ ) {
			$OS = "RHWS5";
			last;
		}

		# ES 6
		# Redhat Enterprise Linux Server 6
		if( /(release)(\s+)(6)/ ) {
			$OS = "RHEL6";
			last;
		}

		# ES 5
		# Redhat Enterprise Linux Server 5
		if( (/(release)(\s+)(5)/)
		||  (/(CentOS)([[:print:]]+)(\s)(5)/) ) {
			$OS = "RHEL5";
			last;
		}

		# EL 4
		# Redhat Enterprise Linux Server 4
		if( (/(release)(\s+)(4)/) 
		||  (/(CentOS)([[:print:]]+)(\s)(4)/) ) {
			$OS = "RHEL4";
			last;
		}

		# Redhat 7.2
		if(/(release)(\s+)(7\.2)/) {
			$OS = "RH72";
			last;
		}
	}
	close(FILE);

	return($OS);
}


#
# Subroutine to run system commands.  This logs an entry.
#
sub mysystem
{
    my $command = $_[0];
    my $returnval = "";

    if ($DEBUG) {
	system("$command >> $LOGFILE 2>> $LOGFILE");
    }
    else {
	system($command);
    }
    $returnval = WEXITSTATUS($?);
    if ($DEBUG) {
        loginfo("$command Returned: $returnval");
    }

    return ($returnval)
}



#
# Log an item to the logfile.
#
sub loginfo
{
        my $message = $_[0];
        my $timestamp = "";

        chomp ($message);
        $timestamp = strftime("%Y-%m-%d %H:%M:%S", localtime());

        open LOGFILE, '>>', "$LOGFILE";
                print LOGFILE "$timestamp <I>: $message\n";
        close LOGFILE;


        return "";
}

#
# Log an error to the logfile.
#
sub logerror
{
        my $message = $_[0];
        my $timestamp = "";

        chomp ($message);
        $timestamp = strftime("%Y-%m-%d %H:%M:%S", localtime());

        open LOGFILE, '>>', "$LOGFILE";
                print LOGFILE "$timestamp <E>: $message\n";
        close LOGFILE;


        return "";
}


exit(main());


__END__

=pod

=head1 NAME

postpatch.pl - apply custom changes after applying a patch


=head1 VERSION

This documenation refers to version 1.16.


=head1 USAGE

postpatch.pl

postpatch.pl B<--version>

postpatch.pl B<--help>

postpatch.pl B<--daisy>

postpatch.pl B<--daisydir=path>

postpatch.pl B<--convert-to-elavon>

postpatch.pl B<--install-ftd-files>


=head1 OPTIONS

=over 4

=item B<--version>

Output the version number of the script and exit.

=item B<--help>

Output a short help message and exit.

=item B<--daisy>

If the directory of the default Daisy database directory "/d/daisy"
does not exist or the command line option "--daisy" is not specified,
the script outputs an error message to stdout and
exits with an exit status of 2.

=item B<--daisydir=path>

Change the value of the default Daisy database directory to that
specified as C<path>.

=item B<--convert-to-elavon>

Instead of performing the typical post patch customizations,
do everything necessary to convert the Daisy system from using
the Global credit card payment system to Elavon.

=item B<--install-ftd-files>

When performing a post patch procedure, if this command line option
is specified, the normal check for the shopcode of the system on the
list of subscribers to the Daisy FTD files will be skipped and the Daisy
FTD files will be installed.

=back


=head1 DESCRIPTION

The usual patch procedure for a Teleflora Daisy POS
involves first running the "applypatch.pl" script to apply a "patch",
i.e. a set of files and contained in a compressed tar archive.
One of the files in the "patch" is the "postpatch.pl" script.
It's behavior is customized for each "patch".
After the files from the patch are applied
(really just copied into place after saving any old instances
of files in the patch in the backup directory), then
this "postpatch.pl" script is run.
Both scripts are run on each of any existing Daisy database dirs.
Usually, the scripts are run from within the context of an
Altiris push, but it can be downloaded to a system and
kicked off manually.


=head1 FILES

=over 4

=item B</d/daisy/config/applypatch.config>

The configuration file for the apply patch process.
Currently, the only variable that may be defined in the config file
is "qa-shopcode=nnnnnnnn" where "nnnnnnnn" represents a shopcode.
There may be zero or more of these lines in the config file.

=item B<daisy-ftd-subs.txt>

This file contains the list of Daisy FTD subscriber shopcodes.
When using the "applypatch" technology for distributing a new
set of electronic directory files, it is one of the files in the
the patch file.

If the shopcode of the system that the patch is running on
does NOT appear in this file, then a message will be logged and
the FTD files will NOT be installed.

However, if the shopcode of the system is defined in the
config file via the "qa-shopcode" variable,
then the check for appropriate shopcode will pass and
the FTD files will be installed.

=item B</d/daisy/log/RTI-Patches.log>

The log file that all log messages are written to.

=item B<backups>

Each Daisy database directory has a directory named "backups" which
is where old instances of files in the "patch" are copied before
being replaced with the new instances.

=back


=head1 DIAGNOSTICS

=over 4

=item Exit status 0

Successful completion or when the "--version" and "--help"
command line options are specified.

=item Exit status 1

If the path to the Daisy database directory to "post patch" is NOT
specified as the first argument on the command line.
If the time stamp is NOT
specified as the second argument on the command line.

=item Exit status 2

If the default Daisy database directory does not exist.

=item Exit status 9

The conversion failed because Daisy could not be stopped.

=item Exit status 10

The conversion failed because Daisy could not be restarted.

=item Exit status 11

If the conversion to Elavon failed.

=back


=head1 SEE ALSO


=cut
